# -*- coding: utf-8 -*-
"""Analyzer module for computing aggregated results from evaluator outputs."""

from .base_analyzer import AnalysisResult, BaseAnalyzer

__all__ = [
    "AnalysisResult",
    "BaseAnalyzer",
]
